import java.util.InputMismatchException;
import java.io.Serializable;
// Author: F. de la Parra
// Date  : January, 2019
// Description:
// Student class serves as a "template" to create objects of type
// Student in class ClassList. It defines two constructor methods
// with different nummbers of arguments and an deprecated method 
// called "description". Mutator and accessor methods provide a 
// common interface to the attributes encapsulated in this class.
// Running the javadoc tool on this class allows the extraction of 
// /** text */ comments and method parameter descriptions into a 
// web page compatible with the stadard API online format for 
// documenting packages and classes.  


// Javadoc @tags
//
// @param ParameterName ParameterDescription
// @return ValueReturnedDescription
// @throws ExceptionType Explanation
// @deprecated
// @see PackageName.ClassName
// @author AuthorName
// @version VersionInformation

/** Class Student is a template to generate 'student objects' that encapsulate
	personal and academic data of university students, as well as the methods to
	update it.
	
	@author F. de la Parra
	@version 1.0
*/

public class Student implements Serializable {
	// Student's basic personal data
	private int studentNumber;
	private String firstName;
	private String lastName;
	private int programYear;
	
	// Student's additional academic data
	private int startYear;
	private boolean doubleMajor;
	private double gpa;
	
	/**
     Full parameter constructor.
     @param studentNumber The 8-digit identification number assigned to a student.
     @param firstName The student's first name.
     @param lastName The student's last name.
     @param programYear The year in an undergraduate program (1, 2, 3, 4).
	 @param startYear The year a student started in an undergraduate program (YYYY).
	 @param doubleMajor Indicates whether a student is registered in a dual program.
     @param gpa The student's current gpa.
     @throws InputMismatchException If student number is not legal.
     */

	public Student(int studentNumber, String firstName, String lastName, int programYear,
			int startYear, boolean doubleMajor, double gpa) throws InputMismatchException {
		
		try {
			setStudentNumber(studentNumber);
		}
		catch (InputMismatchException e) {
			System.out.println("Invalid student number! Program aborted!");
			System.exit(0);
		}
		
		this.firstName = firstName;
		this.lastName = lastName;
		this.programYear = programYear;
		this.startYear = startYear;
		this.doubleMajor = doubleMajor;
		this.gpa = gpa;
	}
	
		/**
     * Basic personal data constructor.
     * @param studentNumber The 8-digit identification number assigned to a student.
     * @param firstName The student's first name.
     * @param lastName The student's last name.
     * @param programYear Year in a program (1 - 4).
     * @throws InputMismatchException If student number is not legal.
     */

	public Student(int studentNumber, String firstName, String lastName, int programYear)
			throws InputMismatchException {
		
		try {
			setStudentNumber(studentNumber);
		}
		catch (InputMismatchException e) {
			System.out.println("Invalid student number! Program aborted!");
			System.exit(0);
		}
		
		this.firstName = firstName;
		this.lastName = lastName;
		this.programYear = programYear;
	}
	
	
	// Miscellaneous methods
	
	/** Method description builds a message about a student.
		
		@return A message identifying a student by name and year in a program.
		@deprecated This method is being phased out.
		@see java.util.Formatter 
	*/
	
	// The @Deprecated annotation indicates to the java compiler that the
	// following method has been deprecated.
	@Deprecated
	public String description() {
		String allText = "Student " + this.firstName + " " + 
		                 this.lastName + " is in programYear " + this.programYear;
		return allText;
	}
	
	// Student mutator methods
	
	/** Student number initialization method. It throws an InputMismatchException
	exception for an invalid student number.
	
	@param studentNumber The 8-digit identification number assigned to a student.
	@return A 'true' status for a valid student number. It returns 'false' otherwise.
	@throws InputMismatchException Invalid student number exception.
	*/
	
	public boolean setStudentNumber(int studentNumber) throws InputMismatchException {
		
		boolean status = false;
		if (studentNumber >= 10000000 && studentNumber < 99999999)
			this.studentNumber = studentNumber;
		else
			throw new InputMismatchException("Invalid student number");
		return true;
	}

	/** Method setStartYear updates the year a student started in an undergraduate program.
		
		@param startYear (YYYY) a student started an undergraduate program.
	*/
	
	public void setStartYear(int startYear) {
		this.startYear = startYear;
	}

	/** Method setDoubleMajor updates a boolean flag indicating whether a student
		is registered in a double major program or not.
		
		@param  doubleMajor An indicator of whether a student is registered in a double-major or not. 
	*/	

	public void setDoubleMajor(boolean doubleMajor) {
		this.doubleMajor = doubleMajor;
	}

	/** Method setGpa updates a student's grade point average.
	
		@param gpa  A student's current grade point average.
	*/	

	public void setGpa(double gpa) {
		this.gpa = gpa;
	}
	
	// Student accessor methods
	
	/** Method getStudentNumber returns a student's identification number.
		
		@return A student's identification number.
	*/
	
	public int getStudentNumber() {
		return this.studentNumber;
	}
	
	/** Method getFirstName returns a student's first name.
		
		@return A student's first name.
	*/
	
	public String getFirstName() {
		return this.firstName;
	}
	
	/** Method getLastName returns a student's last name.
		
		@return A student's last name.
	*/

	public String getLastName() {
		return this.lastName;
	}
	
	/** Method getStartYear returns the programYear a student (object)
		started in an undergraduate program.
		
		@return Year (YYYY) a student started an undergraduate program.
	*/
	
	public int getStartYear() {
		return this.startYear;
	}
	
	/** Method getDoubleMajor returns a boolean flag indicating whether a student
		is registered in a double major program or not.
		
		@return  A student's double-major indicator (true or false).
	*/
	
	public boolean getDoubleMajor() {
		return this.doubleMajor;
	}
	
	/** Method getGpa returns a student's gpa.
		
		@return  A student's grade point average.
	*/
	public double getGpa() {
		return this.gpa;
	}
}